clc,clear all, close all

% Add path of example
addpath('./Square');
% load geometry
load coordinates.dat; 
eval('load elements.dat;','elements3=[];');
eval('load neumann.dat;' ,'neumann=[];');
load dirichlet.dat;

%*** prerefine mesh
[coordinates,elements,dirichlet,neumann] = ...
                          refine(coordinates,elements,dirichlet,neumann);
beta = [100,0];
gamma = 3;

nSteps = 3;
alphaRand = zeros(1,nSteps);
betaRand  = zeros(1,nSteps);
alphaEW   = zeros(1,nSteps);
betaEW    = zeros(1,nSteps);

for j=1:nSteps
  %*** refine mesh
  [coordinates,elements,dirichlet,neumann] = ...
                          refine(coordinates,elements,dirichlet,neumann);

  nE = size(elements,1);
  nC = size(coordinates,1);
  dof(j) = nC;
  dir = unique(dirichlet);

  %*** First vertex of elements and corresponding edge vectors 
  c1 = coordinates(elements(:,1),:);
  d21 = coordinates(elements(:,2),:) - c1;
  d31 = coordinates(elements(:,3),:) - c1;
  %*** Vector of element areas 4*|T|
  area4 = 2*(d21(:,1).*d31(:,2)-d21(:,2).*d31(:,1));

  %*** matrix for diffusion A
  I = reshape(elements(:,[1 2 3 1 2 3 1 2 3])',9*nE,1);
  J = reshape(elements(:,[1 1 1 2 2 2 3 3 3])',9*nE,1);
  a = (sum(d21.*d31,2)./area4)';
  b = (sum(d31.*d31,2)./area4)';
  c = (sum(d21.*d21,2)./area4)';
  A = [-2*a+b+c;a-b;a-c;a-b;b;-a;a-c;-a;c];
  A = sparse(I(:),J(:),A(:));

  %*** matrix for convection B
  a = - 1/6*(d21-d31)* [-beta(2);beta(1)];
  b = - 1/6* d31*      [-beta(2);beta(1)];
  c =   1/6* d21*      [-beta(2);beta(1)];
  B = [a,b,c,a,b,c,a,b,c]';
  B = sparse(I,J,B(:));

  %*** matrix for reaction M
  a = area4/24;
  b = area4/48;
  M = [a,b,b,b,a,b,b,b,a]';
  M = sparse(I,J,M(:));
  
  %*** set freenodes
  freenodes = setdiff(1:nC, dir);
  %*** matrix for actual problem
  AA = A(freenodes,freenodes)+B(freenodes,freenodes)+gamma*M(freenodes,freenodes);

  %*** matrix for H^1 inner product
  MM = A(freenodes,freenodes)+M(freenodes,freenodes);
  %*** compute constants with random vectors
  

  %*** get coercivity with EWP
  
  %*** get inf-sup with EWP

end

loglog(dof,alphaEW,'-r*',dof, betaEW,'-b*'),hold on
loglog(dof,alphaRand,'-g*',dof , betaRand,'-m*')
hold off




